using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using QuickForm;
using Newtonsoft.Json;
using ScaffoldingExample.Helpers;
using ScaffoldingExample.Models;

namespace ScaffoldingExample.Controllers
{   

    public class ProcessorController : Controller
    {
        const string ControllerName = "Processor";
        private Model1Container db = new Model1Container();
        private string DtVarIdName = QuickFormModel.DtVarIdName(ControllerName);

        //
        // GET: /Processor/

        public ViewResult Index()
        {
			int DTListDisplayLength = QuickFormProject.DTListDisplayLength;
			QuickFormModel model = QuickFormProject.Data["Processor"].forIndexClassifier(DTListDisplayLength);
            
            //Выбранная строка в списке с учетом фильтра по вопросам
            var filteredData = GetDataWithFilter().ToList();
            DataTableHelper.InitDtVarIdName(Session, DtVarIdName,
                () => (filteredData.Count() > 0),
                id => (filteredData.Count(x => x.Id == id) > 0),
                () =>
                {
                    //Поиск элемента с минимальным порядком
                    int minOrder = filteredData.Min(x => x.GetOrder());
                    Processor d = filteredData.FirstOrDefault(x => x.GetOrder() == minOrder);
                    if (d != null)
                    {
                        //Если элемен найден, то возвращается его id
                        int Result = d.Id;
                        return Result;
                    }
                    else
                    {
                        //Если элемент не найден, то возвращается первый элемент в последовательности
                        int Result = filteredData.Min(x => x.Id);                        
                        return Result;
                    }
                }
                );

            //Возврат данных в форму
            return View(model);
        }

        /// <summary>
        /// Получение выборки с учетом возможного фильтра
        /// </summary>
        public IQueryable<Processor> GetDataWithFilter()
        {
            var query = from x in db.Processors
                        select x;

            return query;
        }

        public ActionResult Orderchange(int id, int d)
        {
            var query = GetDataWithFilter();

            //Текущий объект
            Processor currentData = query.Single(x => x.Id == id);

            //Если найден
            if (currentData != null)
            {
                //Установка  номера текущей строки в DataTable
                Session.Add(DtVarIdName, id);

                //Изменение порядка
                Processor pairedOrderElement = new Processor();
                OrderHelper<Processor>.OrderChange(query.ToList(), currentData, d, out pairedOrderElement);

                //Сохранение элементов с измененным порядком, если были изменения
                if (pairedOrderElement != null)
                {
                    db.Entry(pairedOrderElement).State = EntityState.Modified;
                    db.Entry(currentData).State = EntityState.Modified;
                    db.SaveChanges();
                }
            }

            return RedirectToAction("Index");
        }

		public string ListData()
        {
            var data = new DtJson<DtData>();
            var query = GetDataWithFilter();
            foreach(var x in query)
            {
                DtData temp = new DtData()
                {
                    id = x.Id.ToString(),
                    order = x.GetOrder(),
                    text = x.ProcessorName
                };
                data.Add(temp);
            }
            return JsonConvert.SerializeObject(data);
        }

        //
        // GET: /Processor/Create

        public ActionResult Create(int? dtid)
        {
            if (dtid.HasValue)
            {
                //Установка  номера текущей строки в DataTable
                Session.Add(DtVarIdName, dtid.Value);
            }

            Processor currentData = new Processor();
            var query = GetDataWithFilter();
            int newOrder = OrderHelper<Processor>.OrderForNewRecord(query);
            currentData.SetOrder(newOrder);
            return View(currentData);
        } 

        //
        // POST: /Processor/Create

        [HttpPost]
        public ActionResult Create(Processor processor)
        {
            if (
                db.Processors.Any(x => x.ProcessorName.Trim().ToUpper().Equals(processor.ProcessorName.Trim().ToUpper()))
                )
            {
                ModelState.AddModelError("ProcessorName", QuickFormProject.ErrorText_RepeatValue);
            }           

            if (ModelState.IsValid)
            {
                db.Processors.Add(processor);
                db.SaveChanges();

                //Установка  номера текущей строки в DataTable
                Session.Add(DtVarIdName, processor.Id);

                return RedirectToAction("Index");  
            }

			return View(processor);
        }
        
        //
        // GET: /Processor/Edit/5
 
        public ActionResult Edit(int id)
        {
            Processor processor = db.Processors.Single(x => x.Id == id);
            if (processor == null)
            {
                return HttpNotFound();
            }

            //Установка  номера текущей строки в DataTable
            Session.Add(DtVarIdName, id);

			return View(processor);
        }

        //
        // POST: /Processor/Edit/5

        [HttpPost]
        public ActionResult Edit(Processor processor)
        {
            if (
                db.Processors.Any(x => x.ProcessorName.Trim().ToUpper().Equals(processor.ProcessorName.Trim().ToUpper())
									&& x.Id != processor.Id)
                )
            {
                ModelState.AddModelError("ProcessorName", QuickFormProject.ErrorText_RepeatValue);
            }           
			
			if (ModelState.IsValid)
            {
                db.Entry(processor).State = EntityState.Modified;
                db.SaveChanges();

                //Установка  номера текущей строки в DataTable
                Session.Add(DtVarIdName, processor.Id);

                return RedirectToAction("Index");
            }

			return View(processor);
        }

        //
        // GET: /Processor/Delete/5
 
        public ActionResult Delete(int id)
        {
            Processor processor = db.Processors.Single(x => x.Id == id);
            if (processor == null)
            {
                return HttpNotFound();
            }

            //Установка  номера текущей строки в DataTable
            Session.Add(DtVarIdName, id);

			return View(processor);
        }

        //
        // POST: /Processor/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(int id)
        {
            Processor processor = db.Processors.Single(x => x.Id == id);

            //Компенсация порядка при удалении
            var query = GetDataWithFilter();
            List<Processor> elementsToSave;
            OrderHelper<Processor>.OrderDelCompensate(query, processor, out elementsToSave);

            //Сохранение элементов с измененным порядком
            foreach (var x in elementsToSave)
            {
                if (x != null)
                {
                    db.Entry(x).State = EntityState.Modified;
                }
            }
            db.SaveChanges();
            //Удаление объекта
            db.Processors.Remove(processor);
            db.SaveChanges();

            //Установка  номера текущей строки в DataTable
            Session.Add(DtVarIdName, null);

            return RedirectToAction("Index");
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing) {
                db.Dispose();
            }
            base.Dispose(disposing);
        }
    }
}